// include files
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <memory.h>
#include "ptc.h"
#include "eff.h"
#include "eff1.h"
#include "3d1.h"
#include "eff2.h"
#include "eff3.h"
#include "eff4.h"
#include "midasdll.h"


#include "Kreis.dat"
#include "Pismo.dat"

unsigned char Vpismo[320*200]; 
unsigned char pcxpalette[768];
// Vpcxloada function 
#include "pcxload.cpp"



/****************************************************************************\
*
* Function:     void MIDASerror(void)
*
* Description:  Handles a MIDAS error - displays an error message and exits
*
\****************************************************************************/

void MIDASerror(void)
{
    int         error;

    error = MIDASgetLastError();
    printf("\nMIDAS error: %s\n", MIDASgetErrorMessage(error));
    if ( !MIDASclose() )
    {
        printf("\nBIG PANIC! MIDASclose Failed: %s\n", MIDASgetErrorMessage(
            MIDASgetLastError()));
    }
    exit(EXIT_FAILURE);
}


int APIENTRY WinMain(HINSTANCE hInst,HINSTANCE hPrevInst,LPSTR lpCmdLine,int nCmdShow)
{
  MIDASmodule module;
  MIDASmodulePlayHandle playHandle;
  MIDASplayStatus moduleStatus;
  double begtime;
  double nowtime;
  //FILE* debugfile;
 
  // debugfile=fopen("debug.out","wt");
  // Initialize MIDAS and start background playback 	   
 if (! MIDASstartup()) 
       MIDASerror(); 
 if (! MIDASconfig())
       MIDASerror();   
 if (! MIDASinit() )
       MIDASerror();
 if ( !MIDASstartBackgroundPlay(0))
       MIDASerror();
 if ( (module = MIDASloadModule("sulfur.xm")) == NULL )
       MIDASerror();
    playHandle = MIDASplayModule(module, TRUE);
 try{
  //effects 
  eff1* effect1=new(eff1);
  eff2* effect2=new(eff2);
  eff3* effect3=new(eff3);
  eff4* effect4=new(eff4);
    
  bool breakintro=false;
  int framecnt=0;
  Format format(8);
  Palette* p_palette;
  Console console;
  Timer timer;

  
  //for charakters
  Vpcxloada(320*200,(char*)Vpismo,(char *)pismo,sizeof(pismo)); 

  //back buffer
  Surface surface(320,200,format);

  // set console options
  console.option("windowed output");

  console.open("Sulfur15CH - Optik 1999",320,200,format);
  // generate palette
  effect1->genpal();
  effect2->genpal();
  effect3->genpal();
  effect4->genpal();
  timer.start();

  p_palette=effect1->getpal();
  // set console palette
  console.palette(*p_palette);
  // set surface palette
  surface.palette(*p_palette);
  while (!breakintro){
   begtime=timer.time();
   //fprintf(debugfile,"%f \n",begtime);
   if (effect1->ESCkey(console))
     breakintro=true;   	  
   // lock surface in this 
   effect1->setsurface(&surface);
   //write one picture into surface
   effect1->play();
   // unlock surface
   surface.unlock();
   // setup copy area
   Area area(0,0,320,200);
   // copy surface to console
   surface.copy(console,area,area);
   // update console
   console.update();
   surface.clear();
   MIDASgetPlayStatus(playHandle,&moduleStatus);
   if (moduleStatus.position>10) {
	   break;
   }
   do{
    nowtime=timer.time();
   }while ((nowtime-begtime)<(0.025));
   //fprintf(debugfile,"%f \n",nowtime);
  }
  framecnt=0;
  
  p_palette=effect2->getpal();
  console.palette(*p_palette);
  surface.palette(*p_palette);
  while (!breakintro){
   begtime=timer.time();
   if (effect2->ESCkey(console))
     breakintro=true;   	  
   effect2->setsurface(&surface);
   effect2->play();
   surface.unlock();
   Area area(0,0,320,200);
   surface.copy(console,area,area);
   console.update();
   surface.clear();
   MIDASgetPlayStatus(playHandle,&moduleStatus);
   //fprintf(debugfile,"eff2 pos %d\n",moduleStatus.position);
   if (moduleStatus.position>28) {
	   break;
   }
   do{
    nowtime=timer.time();
   }while ((nowtime-begtime)<(0.025));
  }
   
 
  framecnt=0;
  p_palette=effect3->getpal();
  console.palette(*p_palette);
  surface.palette(*p_palette);
  while (!breakintro){
   begtime=timer.time();
   if (effect3->ESCkey(console))
   breakintro=true;   	  
   effect3->setsurface(&surface);
   effect3->play();
   surface.unlock();
   Area area(0,0,320,200);
   surface.copy(console,area,area);
   console.update();
   surface.clear();
   MIDASgetPlayStatus(playHandle,&moduleStatus);
   if (moduleStatus.position>44) break; 
   do{
    nowtime=timer.time();
   }while ((nowtime-begtime)<(0.037));
  }

  framecnt=0;
  p_palette=effect4->getpal();
  console.palette(*p_palette);
  surface.palette(*p_palette);
  while (!breakintro && framecnt++<1500){
   begtime=timer.time();
   if (effect4->ESCkey(console))
     breakintro=true;   	  
   effect4->setsurface(&surface);
   effect4->play();
   surface.unlock();
   Area area(0,0,320,200);
   surface.copy(console,area,area);
   console.update();
   surface.clear();
   MIDASgetPlayStatus(playHandle,&moduleStatus);
   if (moduleStatus.position>60) break; 
   do{
    nowtime=timer.time();
   }while ((nowtime-begtime)<(0.035));
  }

  
  delete(effect1);
  delete(effect2);
  delete(effect3);
  delete(effect4);

  // close console
  console.close();
  timer.stop();
 }
 catch (Error &error)
 {
   // report error
   error.report();
 }
 /* Stop MIDAS: */
 if (!MIDASstopModule(playHandle))
        MIDASerror();
 if (!MIDASfreeModule(module))
       MIDASerror();
 if ( !MIDASstopBackgroundPlay()) 
       MIDASerror();
 if ( !MIDASclose())
       MIDASerror();
 //fclose(debugfile); 
 // exit
 return 0;
}
